@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (0, 12, 12)
-CODENAME = "fil\033[33med"
-BUILD_DT = (2021, 8, 6)
+VERSION = (0, 13, 12)
+CODENAME = "future-proof"
+BUILD_DT = (2021, 9, 1)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
