@@ -97,13 +97,11 @@ function up2k_flagbus() {
         }
     };
     var do_take = function (now) {
-        //dbg('*', 'do_take');
         tx(now, "have");
         flag.owner = [flag.id, now];
         flag.ours = true;
     };
     var do_want = function (now) {
-        //dbg('*', 'do_want');
         tx(now, "want");
     };
     flag.take = function (now) {
@@ -135,15 +133,25 @@ function up2k_flagbus() {
 
 
 function U2pvis(act, btns) {
-    this.act = act;
-    this.ctr = { "ok": 0, "ng": 0, "bz": 0, "q": 0 };
-    this.tab = [];
-    this.head = 0;
-    this.tail = -1;
-    this.wsz = 3;
+    var r = this;
+    r.act = act;
+    r.ctr = { "ok": 0, "ng": 0, "bz": 0, "q": 0 };
+    r.tab = [];
+    r.head = 0;
+    r.tail = -1;
+    r.wsz = 3;
 
-    this.addfile = function (entry, sz, draw) {
-        this.tab.push({
+    var markup = {
+        '404': '<span class="err">404</span>',
+        'ERROR': '<span class="err">ERROR</span>',
+        'OS-error': '<span class="err">OS-error</span>',
+        'found': '<span class="inf">found</span>',
+        'YOLO': '<span class="inf">YOLO</span>',
+        'done': '<span class="ok">done</span>',
+    };
+
+    r.addfile = function (entry, sz, draw) {
+        r.tab.push({
             "hn": entry[0],
             "ht": entry[1],
             "hp": entry[2],
@@ -155,64 +163,64 @@ function U2pvis(act, btns) {
             "bd": 0,  // bytes done
             "bd0": 0  // upload start
         });
-        this.ctr["q"]++;
+        r.ctr["q"]++;
         if (!draw)
             return;
 
-        this.drawcard("q");
-        if (this.act == "q") {
-            this.addrow(this.tab.length - 1);
+        r.drawcard("q");
+        if (r.act == "q") {
+            r.addrow(r.tab.length - 1);
         }
-        if (this.act == "bz") {
-            this.bzw();
+        if (r.act == "bz") {
+            r.bzw();
         }
     };
 
-    this.is_act = function (card) {
-        if (this.act == "done")
+    r.is_act = function (card) {
+        if (r.act == "done")
             return card == "ok" || card == "ng";
 
-        return this.act == card;
+        return r.act == card;
     }
 
-    this.seth = function (nfile, field, html) {
-        var fo = this.tab[nfile];
+    r.seth = function (nfile, field, html) {
+        var fo = r.tab[nfile];
         field = ['hn', 'ht', 'hp'][field];
         if (fo[field] === html)
             return;
 
         fo[field] = html;
-        if (!this.is_act(fo.in))
+        if (!r.is_act(fo.in))
             return;
 
         var obj = ebi('f{0}{1}'.format(nfile, field.slice(1)));
-        obj.innerHTML = html;
+        obj.innerHTML = field == 'ht' ? (markup[html] || html) : html;
         if (field == 'hp') {
             obj.style.color = '';
             obj.style.background = '';
         }
     };
 
-    this.setab = function (nfile, nblocks) {
+    r.setab = function (nfile, nblocks) {
         var t = [];
         for (var a = 0; a < nblocks; a++)
             t.push(0);
 
-        this.tab[nfile].cb = t;
+        r.tab[nfile].cb = t;
     };
 
-    this.setat = function (nfile, blocktab) {
-        this.tab[nfile].cb = blocktab;
+    r.setat = function (nfile, blocktab) {
+        r.tab[nfile].cb = blocktab;
 
         var bd = 0;
         for (var a = 0; a < blocktab.length; a++)
             bd += blocktab[a];
 
-        this.tab[nfile].bd = bd;
-        this.tab[nfile].bd0 = bd;
+        r.tab[nfile].bd = bd;
+        r.tab[nfile].bd0 = bd;
     };
 
-    this.perc = function (bd, bd0, sz, t0) {
+    r.perc = function (bd, bd0, sz, t0) {
         var td = Date.now() - t0,
             p = bd * 100.0 / sz,
             nb = bd - bd0,
@@ -222,15 +230,15 @@ function U2pvis(act, btns) {
         return [p, s2ms(eta), spd / (1024 * 1024)];
     };
 
-    this.hashed = function (fobj) {
-        var fo = this.tab[fobj.n],
+    r.hashed = function (fobj) {
+        var fo = r.tab[fobj.n],
             nb = fo.bt * (++fo.nh / fo.cb.length),
-            p = this.perc(nb, 0, fobj.size, fobj.t_hashing);
+            p = r.perc(nb, 0, fobj.size, fobj.t_hashing);
 
         fo.hp = '{0}%, {1}, {2} MB/s'.format(
-            p[0].toFixed(2), p[1], p[2].toFixed(2)
+            f2f(p[0], 2), p[1], f2f(p[2], 2)
         );
-        if (!this.is_act(fo.in))
+        if (!r.is_act(fo.in))
             return;
 
         var obj = ebi('f{0}p'.format(fobj.n)),
@@ -241,31 +249,31 @@ function U2pvis(act, btns) {
         obj.style.background = 'linear-gradient(90deg, #025, #06a ' + o1 + '%, #09d ' + o2 + '%, #333 ' + o3 + '%, #333 99%, #777)';
     };
 
-    this.prog = function (fobj, nchunk, cbd) {
-        var fo = this.tab[fobj.n],
+    r.prog = function (fobj, nchunk, cbd) {
+        var fo = r.tab[fobj.n],
             delta = cbd - fo.cb[nchunk];
 
         fo.cb[nchunk] = cbd;
         fo.bd += delta;
 
-        var p = this.perc(fo.bd, fo.bd0, fo.bt, fobj.t_uploading);
+        var p = r.perc(fo.bd, fo.bd0, fo.bt, fobj.t_uploading);
         fo.hp = '{0}%, {1}, {2} MB/s'.format(
-            p[0].toFixed(2), p[1], p[2].toFixed(2)
+            f2f(p[0], 2), p[1], f2f(p[2], 2)
         );
 
-        if (!this.is_act(fo.in))
+        if (!r.is_act(fo.in))
             return;
 
         var obj = ebi('f{0}p'.format(fobj.n)),
             o1 = p[0] - 2, o2 = p[0] - 0.1, o3 = p[0];
 
-        if (!obj) { //} || true) {
+        if (!obj) {
             var msg = [
-                "act", this.act,
+                "act", r.act,
                 "in", fo.in,
-                "is_act", this.is_act(fo.in),
-                "head", this.head,
-                "tail", this.tail,
+                "is_act", r.is_act(fo.in),
+                "head", r.head,
+                "tail", r.tail,
                 "nfile", fobj.n,
                 "name", fobj.name,
                 "sz", fobj.size,
@@ -283,12 +291,12 @@ function U2pvis(act, btns) {
             for (var a = 0, aa = ds.length; a < aa; a++) {
                 var id = ds[a].parentNode.getAttribute('id').slice(1, -1);
                 console.log("dom %d/%d = [%s] in(%s) is_act(%s) %s",
-                    a, aa, id, this.tab[id].in, this.is_act(fo.in), ds[a].textContent);
+                    a, aa, id, r.tab[id].in, r.is_act(fo.in), ds[a].textContent);
             }
 
-            for (var a = 0, aa = this.tab.length; a < aa; a++)
-                if (this.is_act(this.tab[a].in))
-                    console.log("tab %d/%d = sz %s", a, aa, this.tab[a].bt);
+            for (var a = 0, aa = r.tab.length; a < aa; a++)
+                if (r.is_act(r.tab[a].in))
+                    console.log("tab %d/%d = sz %s", a, aa, r.tab[a].bt);
 
             throw new Error('see console');
         }
@@ -298,27 +306,27 @@ function U2pvis(act, btns) {
         obj.style.background = 'linear-gradient(90deg, #050, #270 ' + o1 + '%, #4b0 ' + o2 + '%, #333 ' + o3 + '%, #333 99%, #777)';
     };
 
-    this.move = function (nfile, newcat) {
-        var fo = this.tab[nfile],
+    r.move = function (nfile, newcat) {
+        var fo = r.tab[nfile],
             oldcat = fo.in,
-            bz_act = this.act == "bz";
+            bz_act = r.act == "bz";
 
         if (oldcat == newcat)
             return;
 
         fo.in = newcat;
-        this.ctr[oldcat]--;
-        this.ctr[newcat]++;
-        this.drawcard(oldcat);
-        this.drawcard(newcat);
-        if (this.is_act(newcat)) {
-            this.tail = Math.max(this.tail, nfile + 1);
+        r.ctr[oldcat]--;
+        r.ctr[newcat]++;
+        r.drawcard(oldcat);
+        r.drawcard(newcat);
+        if (r.is_act(newcat)) {
+            r.tail = Math.max(r.tail, nfile + 1);
             if (!ebi('f' + nfile))
-                this.addrow(nfile);
+                r.addrow(nfile);
         }
-        else if (this.is_act(oldcat)) {
-            while (this.head < Math.min(this.tab.length, this.tail) && this.precard[this.tab[this.head].in])
-                this.head++;
+        else if (r.is_act(oldcat)) {
+            while (r.head < Math.min(r.tab.length, r.tail) && r.precard[r.tab[r.head].in])
+                r.head++;
 
             if (!bz_act) {
                 var tr = ebi("f" + nfile);
@@ -328,10 +336,10 @@ function U2pvis(act, btns) {
         else return;
 
         if (bz_act)
-            this.bzw();
+            r.bzw();
     };
 
-    this.bzw = function () {
+    r.bzw = function () {
         var first = QS('#u2tab>tbody>tr:first-child');
         if (!first)
             return;
@@ -340,93 +348,93 @@ function U2pvis(act, btns) {
         first = parseInt(first.getAttribute('id').slice(1));
         last = parseInt(last.getAttribute('id').slice(1));
 
-        while (this.head - first > this.wsz) {
+        while (r.head - first > r.wsz) {
             var obj = ebi('f' + (first++));
             if (obj)
                 obj.parentNode.removeChild(obj);
         }
-        while (last - this.tail < this.wsz && last < this.tab.length - 2) {
+        while (last - r.tail < r.wsz && last < r.tab.length - 2) {
             var obj = ebi('f' + (++last));
             if (!obj)
-                this.addrow(last);
+                r.addrow(last);
         }
     };
 
-    this.drawcard = function (cat) {
+    r.drawcard = function (cat) {
         var cards = QSA('#u2cards>a>span');
 
         if (cat == "q") {
-            cards[4].innerHTML = this.ctr[cat];
+            cards[4].innerHTML = r.ctr[cat];
             return;
         }
         if (cat == "bz") {
-            cards[3].innerHTML = this.ctr[cat];
+            cards[3].innerHTML = r.ctr[cat];
             return;
         }
 
-        cards[2].innerHTML = this.ctr["ok"] + this.ctr["ng"];
+        cards[2].innerHTML = r.ctr["ok"] + r.ctr["ng"];
 
         if (cat == "ng") {
-            cards[1].innerHTML = this.ctr[cat];
+            cards[1].innerHTML = r.ctr[cat];
         }
         if (cat == "ok") {
-            cards[0].innerHTML = this.ctr[cat];
+            cards[0].innerHTML = r.ctr[cat];
         }
     };
 
-    this.changecard = function (card) {
-        this.act = card;
-        this.precard = has(["ok", "ng", "done"], this.act) ? {} : this.act == "bz" ? { "ok": 1, "ng": 1 } : { "ok": 1, "ng": 1, "bz": 1 };
-        this.postcard = has(["ok", "ng", "done"], this.act) ? { "bz": 1, "q": 1 } : this.act == "bz" ? { "q": 1 } : {};
-        this.head = -1;
-        this.tail = -1;
+    r.changecard = function (card) {
+        r.act = card;
+        r.precard = has(["ok", "ng", "done"], r.act) ? {} : r.act == "bz" ? { "ok": 1, "ng": 1 } : { "ok": 1, "ng": 1, "bz": 1 };
+        r.postcard = has(["ok", "ng", "done"], r.act) ? { "bz": 1, "q": 1 } : r.act == "bz" ? { "q": 1 } : {};
+        r.head = -1;
+        r.tail = -1;
         var html = [];
-        for (var a = 0; a < this.tab.length; a++) {
-            var rt = this.tab[a].in;
-            if (this.is_act(rt)) {
-                html.push(this.genrow(a, true));
+        for (var a = 0; a < r.tab.length; a++) {
+            var rt = r.tab[a].in;
+            if (r.is_act(rt)) {
+                html.push(r.genrow(a, true));
 
-                this.tail = a;
-                if (this.head == -1)
-                    this.head = a;
+                r.tail = a;
+                if (r.head == -1)
+                    r.head = a;
             }
         }
-        if (this.head == -1) {
-            for (var a = 0; a < this.tab.length; a++) {
-                var rt = this.tab[a].in;
-                if (this.precard[rt]) {
-                    this.head = a + 1;
-                    this.tail = a;
+        if (r.head == -1) {
+            for (var a = 0; a < r.tab.length; a++) {
+                var rt = r.tab[a].in;
+                if (r.precard[rt]) {
+                    r.head = a + 1;
+                    r.tail = a;
                 }
-                else if (this.postcard[rt]) {
-                    this.head = a;
-                    this.tail = a - 1;
+                else if (r.postcard[rt]) {
+                    r.head = a;
+                    r.tail = a - 1;
                     break;
                 }
             }
         }
 
-        if (this.head < 0)
-            this.head = 0;
+        if (r.head < 0)
+            r.head = 0;
 
         if (card == "bz") {
-            for (var a = this.head - 1; a >= this.head - this.wsz && a >= 0; a--) {
-                html.unshift(this.genrow(a, true).replace(/><td>/, "><td>a "));
+            for (var a = r.head - 1; a >= r.head - r.wsz && a >= 0; a--) {
+                html.unshift(r.genrow(a, true).replace(/><td>/, "><td>a "));
             }
-            for (var a = this.tail + 1; a <= this.tail + this.wsz && a < this.tab.length; a++) {
-                html.push(this.genrow(a, true).replace(/><td>/, "><td>b "));
+            for (var a = r.tail + 1; a <= r.tail + r.wsz && a < r.tab.length; a++) {
+                html.push(r.genrow(a, true).replace(/><td>/, "><td>b "));
             }
         }
         ebi('u2tab').tBodies[0].innerHTML = html.join('\n');
     };
 
-    this.genrow = function (nfile, as_html) {
-        var r = this.tab[nfile],
+    r.genrow = function (nfile, as_html) {
+        var row = r.tab[nfile],
             td1 = '<td id="f' + nfile,
             td = '</td>' + td1,
-            ret = td1 + 'n">' + r.hn +
-                td + 't">' + r.ht +
-                td + 'p" class="prog">' + r.hp + '</td>';
+            ret = td1 + 'n">' + row.hn +
+                td + 't">' + (markup[row.ht] || row.ht) +
+                td + 'p" class="prog">' + row.hp + '</td>';
 
         if (as_html)
             return '<tr id="f' + nfile + '">' + ret + '</tr>';
@@ -437,40 +445,50 @@ function U2pvis(act, btns) {
         return obj;
     };
 
-    this.addrow = function (nfile) {
-        var tr = this.genrow(nfile);
+    r.addrow = function (nfile) {
+        var tr = r.genrow(nfile);
         ebi('u2tab').tBodies[0].appendChild(tr);
     };
 
-    var that = this;
     btns = QSA(btns + '>a[act]');
     for (var a = 0; a < btns.length; a++) {
         btns[a].onclick = function (e) {
             ev(e);
             var newtab = this.getAttribute('act');
-            for (var b = 0; b < btns.length; b++) {
-                btns[b].className = (
-                    btns[b].getAttribute('act') == newtab) ? 'act' : '';
+            function go() {
+                for (var b = 0; b < btns.length; b++) {
+                    btns[b].className = (
+                        btns[b].getAttribute('act') == newtab) ? 'act' : '';
+                }
+                r.changecard(newtab);
             }
-            that.changecard(newtab);
+            var nf = r.ctr[newtab];
+            if (nf === undefined)
+                nf = r.ctr["ok"] + r.ctr["ng"];
+
+            if (nf < 9000)
+                return go();
+
+            modal.confirm('about to show ' + nf + ' files\n\nthis may crash your browser, are you sure?', go, null);
         };
     }
 
-    this.changecard(this.act);
+    r.changecard(r.act);
 }
 
 
-function fsearch_explain(e) {
-    ev(e);
-    if (!has(perms, 'write'))
-        return alert('your access to this folder is Read-Only\n\n' + (acct == '*' ? 'you are currently not logged in' : 'you are currently logged in as ' + acct));
+function fsearch_explain(n) {
+    if (n)
+        return toast.inf(60, 'your access to this folder is Read-Only\n\n' + (acct == '*' ? 'you are currently not logged in' : 'you are currently logged in as "' + acct + '"'));
 
-    alert('you are currently in file-search mode\n\nswitch to upload-mode by clicking the green magnifying glass (next to the big yellow search button), and then refresh\n\nsorry');
+    if (bcfg_get('fsearch', false))
+        return toast.inf(60, 'you are currently in file-search mode\n\nswitch to upload-mode by clicking the green magnifying glass (next to the big yellow search button), and then refresh\n\nsorry');
+
+    return toast.inf(60, 'refresh the page and try again, it should work now');
 }
 
 
 function up2k_init(subtle) {
-    // show modal message
     function showmodal(msg) {
         ebi('u2notbtn').innerHTML = msg;
         ebi('u2btn').style.display = 'none';
@@ -478,7 +496,6 @@ function up2k_init(subtle) {
         ebi('u2conf').style.opacity = '0.5';
     }
 
-    // hide modal message
     function unmodal() {
         ebi('u2notbtn').style.display = 'none';
         ebi('u2btn').style.display = 'block';
@@ -495,7 +512,6 @@ function up2k_init(subtle) {
         // chrome<37 firefox<34 edge<12 opera<24 safari<7
         shame = 'your browser is impressively ancient';
 
-    // upload ui hidden by default, clicking the header shows it
     var got_deps = false;
     function init_deps() {
         if (!got_deps && !subtle && !window.asmCrypto) {
@@ -504,7 +520,7 @@ function up2k_init(subtle) {
             import_js('/.cpr/deps/' + fn, unmodal);
 
             if (is_https)
-                ebi('u2foot').innerHTML = shame + ' so <em>this</em> uploader will do like 500kB/s at best';
+                ebi('u2foot').innerHTML = shame + ' so <em>this</em> uploader will do like 500 KiB/s at best';
             else
                 ebi('u2foot').innerHTML = 'seems like ' + shame + ' so do that if you want more performance <span style="color:#' +
                     (sha_js == 'ac' ? 'c84">(expecting 20' : '8a5">(but dont worry too much, expect 100') + ' MiB/s)</span>';
@@ -512,11 +528,9 @@ function up2k_init(subtle) {
         got_deps = true;
     }
 
-    // show uploader if the user only has write-access
     if (perms.length && !has(perms, 'read'))
         goto('up2k');
 
-    // shows or clears a message in the basic uploader ui
     function setmsg(msg, type) {
         if (msg !== undefined) {
             ebi('u2err').setAttribute('class', type);
@@ -534,13 +548,11 @@ function up2k_init(subtle) {
         }
     }
 
-    // switches to the basic uploader with msg as error message
     function un2k(msg) {
         setmsg(msg, 'err');
         return false;
     }
 
-    // handle user intent to use the basic uploader instead
     ebi('u2nope').onclick = function (e) {
         ev(e);
         setmsg(suggest_up2k, 'msg');
@@ -584,8 +596,15 @@ function up2k_init(subtle) {
             "upload": []
         },
         "bytes": {
+            "total": 0,
             "hashed": 0,
-            "uploaded": 0
+            "uploaded": 0,
+            "finished": 0
+        },
+        "time": {
+            "hashing": 0,
+            "uploading": 0,
+            "busy": 0
         }
     };
 
@@ -642,7 +661,7 @@ function up2k_init(subtle) {
         else files = e.target.files;
 
         if (!files || !files.length)
-            return alert('no files selected??');
+            return toast.err(0, 'no files selected??');
 
         more_one_file();
         var bad_files = [],
@@ -706,13 +725,13 @@ function up2k_init(subtle) {
                     match = false;
 
             if (match) {
-                var msg = ['directory iterator got stuck on the following {0} items; good chance your browser is about to spinlock:'.format(missing.length)];
+                var msg = ['directory iterator got stuck on the following {0} items; good chance your browser is about to spinlock:<ul>'.format(missing.length)];
                 for (var a = 0; a < Math.min(20, missing.length); a++)
-                    msg.push(missing[a]);
+                    msg.push('<li>' + esc(missing[a]) + '</li>');
 
-                alert(msg.join('\n-- '));
-                dirs = [];
-                pf = [];
+                return modal.alert(msg.join('') + '</ul>', function () {
+                    read_dirs(rd, [], [], good, bad, spins);
+                });
             }
             spins = 0;
         }
@@ -757,7 +776,6 @@ function up2k_init(subtle) {
                 }
                 ngot += 1;
             });
-            // console.log("ngot: " + ngot);
             if (!ngot) {
                 dirs.shift();
                 rd = null;
@@ -777,16 +795,28 @@ function up2k_init(subtle) {
             if (good_files.length - bad_files.length <= 1 && ANDROID)
                 msg += '\nFirefox-Android has a bug which prevents selecting multiple files. Try selecting one file at a time. For more info, see firefox bug 1456557';
 
-            alert(msg);
+            return modal.alert(msg, function () {
+                gotallfiles(good_files, []);
+            });
         }
 
-        var msg = ['upload these ' + good_files.length + ' files?'];
+        good_files.sort(function (a, b) {
+            a = a[1];
+            b = b[1];
+            return a < b ? -1 : a > b ? 1 : 0;
+        });
+
+        var msg = ['{0} these {1} files?<ul>'.format(fsearch ? 'search' : 'upload', good_files.length)];
         for (var a = 0, aa = Math.min(20, good_files.length); a < aa; a++)
-            msg.push(good_files[a][1]);
+            msg.push('<li>' + esc(good_files[a][1]) + '</li>');
 
-        if (ask_up && !fsearch && !confirm(msg.join('\n')))
-            return;
+        if (ask_up && !fsearch)
+            return modal.confirm(msg.join('') + '</ul>', function () { up_them(good_files); }, null);
 
+        up_them(good_files);
+    }
+
+    function up_them(good_files) {
         var seen = {},
             evpath = get_evpath(),
             draw_each = good_files.length < 50;
@@ -796,22 +826,33 @@ function up2k_init(subtle) {
 
         for (var a = 0; a < good_files.length; a++) {
             var fobj = good_files[a][0],
+                name = good_files[a][1],
+                fdir = evpath,
                 now = Date.now(),
-                lmod = fobj.lastModified || now;
+                lmod = fobj.lastModified || now,
+                ofs = name.lastIndexOf('/') + 1;
+
+            if (ofs) {
+                fdir += url_enc(name.slice(0, ofs));
+                name = name.slice(ofs);
+            }
 
             var entry = {
                 "n": st.files.length,
                 "t0": now,
                 "fobj": fobj,
-                "name": good_files[a][1],
+                "name": name,
                 "size": fobj.size,
                 "lmod": lmod / 1000,
-                "purl": evpath,
+                "purl": fdir,
                 "done": false,
                 "hash": []
             },
                 key = entry.name + '\n' + entry.size;
 
+            if (fsearch)
+                entry.srch = 1;
+
             if (seen[key])
                 continue;
 
@@ -824,6 +865,7 @@ function up2k_init(subtle) {
                 ''
             ], fobj.size, draw_each);
 
+            st.bytes.total += fobj.size;
             st.files.push(entry);
             if (turbo)
                 push_t(st.todo.head, entry);
@@ -862,6 +904,69 @@ function up2k_init(subtle) {
     }
     ebi('u2cleanup').onclick = u2cleanup;
 
+    var etaref = 0, etaskip = 0;
+    function etafun() {
+        var nhash = st.busy.head.length + st.busy.hash.length + st.todo.head.length + st.todo.hash.length,
+            nsend = st.busy.upload.length + st.todo.upload.length,
+            now = Date.now(),
+            td = (now - (etaref || now)) / 1000.0;
+
+        etaref = now;
+        //ebi('acc_info').innerHTML = humantime(st.time.busy) + ' ' + f2f(now / 1000, 1);
+
+        if (!nhash)
+            ebi('u2etah').innerHTML = 'Done ({0}, {1} files)'.format(humansize(st.bytes.hashed), pvis.ctr["ok"] + pvis.ctr["ng"]);
+
+        if (!nsend && !nhash)
+            ebi('u2etau').innerHTML = ebi('u2etat').innerHTML = (
+                'Done ({0}, {1} files)'.format(humansize(st.bytes.uploaded), pvis.ctr["ok"] + pvis.ctr["ng"]));
+
+        if (!st.busy.hash.length && !hashing_permitted())
+            nhash = 0;
+
+        if (!parallel_uploads || !(nhash + nsend) || (flag && flag.owner && !flag.ours))
+            return;
+
+        var t = [];
+        if (nhash) {
+            st.time.hashing += td;
+            t.push(['u2etah', st.bytes.hashed, st.bytes.hashed, st.time.hashing]);
+            if (fsearch)
+                t.push(['u2etat', st.bytes.hashed, st.bytes.hashed, st.time.hashing]);
+        }
+        if (nsend) {
+            st.time.uploading += td;
+            t.push(['u2etau', st.bytes.uploaded, st.bytes.finished, st.time.uploading]);
+        }
+        if ((nhash || nsend) && !fsearch) {
+            if (!st.bytes.finished) {
+                ebi('u2etat').innerHTML = '(preparing to upload)';
+            }
+            else {
+                st.time.busy += td;
+                t.push(['u2etat', st.bytes.finished, st.bytes.finished, st.time.busy]);
+            }
+        }
+        for (var a = 0; a < t.length; a++) {
+            var rem = st.bytes.total - t[a][2],
+                bps = t[a][1] / t[a][3],
+                eta = Math.floor(rem / bps);
+
+            if (t[a][1] < 1024 || t[a][3] < 0.1) {
+                ebi(t[a][0]).innerHTML = '(preparing to upload)';
+                continue;
+            }
+
+            if (etaskip)
+                continue;
+
+            ebi(t[a][0]).innerHTML = '{0}, {1}/s, {2}'.format(
+                humansize(rem), humansize(bps, 1), humantime(eta));
+        }
+        if (++etaskip > 2)
+            etaskip = 0;
+    }
+
     /////
     ////
     ///   actuator
@@ -900,32 +1005,33 @@ function up2k_init(subtle) {
     }
 
     function hashing_permitted() {
+        if (!parallel_uploads)
+            return false;
+
         if (multitask) {
-            var ahead = st.bytes.hashed - st.bytes.uploaded;
+            var ahead = st.bytes.hashed - st.bytes.finished;
             return ahead < 1024 * 1024 * 1024 * 4 &&
                 st.todo.handshake.length + st.busy.handshake.length < 16;
         }
         return handshakes_permitted() && 0 ==
             st.todo.handshake.length +
-            st.busy.handshake.length;
+            st.busy.handshake.length +
+            st.todo.upload.length +
+            st.busy.upload.length;
     }
 
     var tasker = (function () {
-        var tto = null,
-            running = false,
+        var running = false,
             was_busy = false;
 
         function defer() {
             running = false;
-            clearTimeout(tto);
-            tto = setTimeout(taskerd, 100);
         }
 
         function taskerd() {
             if (running)
                 return;
 
-            clearTimeout(tto);
             if (crashed)
                 return defer();
 
@@ -947,11 +1053,40 @@ function up2k_init(subtle) {
 
                     window[(is_busy ? "add" : "remove") +
                         "EventListener"]("beforeunload", warn_uploader_busy);
+
+                    if (!is_busy) {
+                        var k = fsearch ? 'searches' : 'uploads',
+                            ks = fsearch ? 'Search' : 'Upload',
+                            tok = fsearch ? 'successful (found on server)' : 'completed successfully',
+                            tng = fsearch ? 'failed (NOT found on server)' : 'failed, sorry',
+                            ok = pvis.ctr["ok"],
+                            ng = pvis.ctr["ng"],
+                            t = ask_up ? 0 : 10;
+
+                        if (ok && ng)
+                            toast.warn(t, 'Finished, but some {0} failed:\n{1} {2},\n{3} {4}'.format(k, ok, tok, ng, tng));
+                        else if (ok > 1)
+                            toast.ok(t, 'All {1} {0} {2}'.format(k, ok, tok));
+                        else if (ok)
+                            toast.ok(t, '{0} {1}'.format(ks, tok));
+                        else if (ng > 1)
+                            toast.err(t, 'All {1} {0} {2}'.format(k, ng, tng));
+                        else if (ng)
+                            toast.err(t, '{0} {1}'.format(ks, tng));
+
+                        timer.rm(etafun);
+                    }
+                    else {
+                        timer.add(etafun, false);
+                        ebi('u2etas').style.textAlign = 'left';
+                    }
+                    etafun();
                 }
 
                 if (flag) {
                     if (is_busy) {
                         flag.take(now);
+                        ebi('u2flagblock').style.display = flag.ours ? '' : 'block';
                         if (!flag.ours)
                             return defer();
                     }
@@ -1016,7 +1151,7 @@ function up2k_init(subtle) {
                     return defer();
             }
         }
-        taskerd();
+        timer.add(taskerd, true);
         return taskerd;
     })();
 
@@ -1087,7 +1222,6 @@ function up2k_init(subtle) {
     function exec_hash() {
         var t = st.todo.hash.shift();
         st.busy.hash.push(t);
-        st.bytes.hashed += t.size;
         t.bytes_uploaded = 0;
 
         var bpend = 0,
@@ -1113,6 +1247,7 @@ function up2k_init(subtle) {
                 cdr = t.size;
 
             bpend += cdr - car;
+            st.bytes.hashed += cdr - car;
 
             function orz(e) {
                 if (!min_filebuf && nch == 1) {
@@ -1143,11 +1278,11 @@ function up2k_init(subtle) {
                 if (handled) {
                     pvis.move(t.n, 'ng');
                     apop(st.busy.hash, t);
-                    st.bytes.uploaded += t.size;
+                    st.bytes.finished += t.size;
                     return tasker();
                 }
 
-                alert('y o u   b r o k e    i t\nfile: ' + t.name + '\nerror: ' + err);
+                toast.err(0, 'y o u   b r o k e    i t\nfile: ' + esc(t.name + '') + '\nerror: ' + err);
             };
             reader.readAsArrayBuffer(
                 bobslice.call(t.fobj, car, cdr));
@@ -1235,21 +1370,22 @@ function up2k_init(subtle) {
                 }
             }
             apop(st.busy.head, t);
-            if (!ok)
+            if (!ok && !t.srch)
                 return push_t(st.todo.hash, t);
 
             t.done = true;
             st.bytes.hashed += t.size;
-            st.bytes.uploaded += t.size;
-            pvis.seth(t.n, 1, 'YOLO');
+            st.bytes.finished += t.size;
+            pvis.move(t.n, 'bz');
+            pvis.seth(t.n, 1, ok ? 'YOLO' : '404');
             pvis.seth(t.n, 2, "turbo'd");
-            pvis.move(t.n, 'ok');
+            pvis.move(t.n, ok ? 'ok' : 'ng');
         };
         xhr.onload = function (e) {
             try { orz(e); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
         };
 
-        xhr.open('HEAD', t.purl + t.name, true);
+        xhr.open('HEAD', t.purl + uricom_enc(t.name) + '?raw', true);
         xhr.send();
     }
 
@@ -1301,7 +1437,8 @@ function up2k_init(subtle) {
 
                     if (!response || !response.hits || !response.hits.length) {
                         smsg = '404';
-                        msg = 'not found on server <a href="#" onclick="fsearch_explain()" class="fsearch_explain">(explain)</a>';
+                        msg = ('not found on server <a href="#" onclick="fsearch_explain(' +
+                            (has(perms, 'write') ? '0' : '1') + ')" class="fsearch_explain">(explain)</a>');
                     }
                     else {
                         smsg = 'found';
@@ -1319,17 +1456,19 @@ function up2k_init(subtle) {
                     pvis.seth(t.n, 1, smsg);
                     pvis.move(t.n, smsg == '404' ? 'ng' : 'ok');
                     apop(st.busy.handshake, t);
-                    st.bytes.uploaded += t.size;
+                    st.bytes.finished += t.size;
                     t.done = true;
                     tasker();
                     return;
                 }
 
-                if (response.name !== t.name) {
-                    // file exists; server renamed us
-                    console.log("server-rename [" + t.name + "] to [" + response.name + "]");
+                var rsp_purl = url_enc(response.purl);
+                if (rsp_purl !== t.purl || response.name !== t.name) {
+                    // server renamed us (file exists / path restrictions)
+                    console.log("server-rename [" + t.purl + "] [" + t.name + "] to [" + rsp_purl + "] [" + response.name + "]");
+                    t.purl = rsp_purl;
                     t.name = response.name;
-                    pvis.seth(t.n, 0, linksplit(t.purl + t.name).join(' '));
+                    pvis.seth(t.n, 0, linksplit(uricom_dec(t.purl)[0] + t.name).join(' '));
                 }
 
                 var chunksize = get_chunksize(t.size),
@@ -1347,7 +1486,7 @@ function up2k_init(subtle) {
                 for (var a = 0; a < missing.length; a++) {
                     var idx = t.hash.indexOf(missing[a]);
                     if (idx < 0)
-                        return alert('wtf negative index for hash "{0}" in task:\n{1}'.format(
+                        return modal.alert('wtf negative index for hash "{0}" in task:\n{1}'.format(
                             missing[a], JSON.stringify(t)));
 
                     t.postlist.push(idx);
@@ -1358,7 +1497,7 @@ function up2k_init(subtle) {
                 pvis.prog(t, 0, cbd[0]);
 
                 var done = true,
-                    msg = '&#x1f3b7;&#x1f41b;';
+                    msg = 'done';
 
                 if (t.postlist.length) {
                     var arr = st.todo.upload,
@@ -1385,12 +1524,12 @@ function up2k_init(subtle) {
 
                 if (done) {
                     t.done = true;
-                    st.bytes.uploaded += t.size - t.bytes_uploaded;
+                    st.bytes.finished += t.size - t.bytes_uploaded;
                     var spd1 = (t.size / ((t.t_hashed - t.t_hashing) / 1000.)) / (1024 * 1024.),
                         spd2 = (t.size / ((t.t_uploaded - t.t_uploading) / 1000.)) / (1024 * 1024.);
 
                     pvis.seth(t.n, 2, 'hash {0}, up {1} MB/s'.format(
-                        spd1.toFixed(2), spd2.toFixed(2)));
+                        f2f(spd1, 2), isNaN(spd2) ? '--' : f2f(spd2, 2)));
 
                     pvis.move(t.n, 'ok');
                 }
@@ -1418,7 +1557,7 @@ function up2k_init(subtle) {
                     return;
                 }
 
-                st.bytes.uploaded += t.size;
+                st.bytes.finished += t.size;
                 if (rsp.indexOf('partial upload exists') !== -1 ||
                     rsp.indexOf('file already exists') !== -1) {
                     err = rsp;
@@ -1436,7 +1575,7 @@ function up2k_init(subtle) {
                     tasker();
                     return;
                 }
-                alert("server broke; hs-err {0} on file [{1}]:\n".format(
+                toast.err(0, "server broke; hs-err {0} on file [{1}]:\n".format(
                     xhr.status, t.name) + (
                         (xhr.response && xhr.response.err) ||
                         (xhr.responseText && xhr.responseText) ||
@@ -1453,10 +1592,10 @@ function up2k_init(subtle) {
             "lmod": t.lmod,
             "hash": t.hash
         };
-        if (fsearch)
+        if (t.srch)
             req.srch = 1;
 
-        xhr.open('POST', t.purl + 'handshake.php', true);
+        xhr.open('POST', t.purl, true);
         xhr.responseType = 'text';
         xhr.send(JSON.stringify(req));
     }
@@ -1489,6 +1628,7 @@ function up2k_init(subtle) {
             var txt = ((xhr.response && xhr.response.err) || xhr.responseText) + '';
             if (xhr.status == 200) {
                 pvis.prog(t, npart, cdr - car);
+                st.bytes.finished += cdr - car;
                 st.bytes.uploaded += cdr - car;
                 t.bytes_uploaded += cdr - car;
             }
@@ -1496,7 +1636,7 @@ function up2k_init(subtle) {
                 console.log("ignoring dupe-segment error", t);
             }
             else {
-                alert("server broke; cu-err {0} on file [{1}]:\n".format(
+                toast.err(0, "server broke; cu-err {0} on file [{1}]:\n".format(
                     xhr.status, t.name) + (txt || "no further information"));
                 return;
             }
@@ -1524,7 +1664,7 @@ function up2k_init(subtle) {
                 console.log('chunkpit onerror, retrying', t);
                 do_send();
             };
-            xhr.open('POST', t.purl + 'chunkpit.php', true);
+            xhr.open('POST', t.purl, true);
             xhr.setRequestHeader("X-Up2k-Hash", t.hash[npart]);
             xhr.setRequestHeader("X-Up2k-Wark", t.wark);
             xhr.setRequestHeader('Content-Type', 'application/octet-stream');
@@ -1544,7 +1684,7 @@ function up2k_init(subtle) {
 
     function onresize(e) {
         var bar = ebi('ops'),
-            wpx = innerWidth,
+            wpx = window.innerWidth,
             fpx = parseInt(getComputedStyle(bar)['font-size']),
             wem = wpx * 1.0 / fpx,
             wide = wem > 54,
@@ -1556,6 +1696,7 @@ function up2k_init(subtle) {
             parent.appendChild(btn);
             ebi('u2conf').setAttribute('class', wide ? 'has_btn' : '');
             ebi('u2cards').setAttribute('class', wide ? 'w' : '');
+            ebi('u2etaw').setAttribute('class', wide ? 'w' : '');
         }
     }
     window.addEventListener('resize', onresize);
@@ -1643,7 +1784,7 @@ function up2k_init(subtle) {
 
     function draw_turbo() {
         var msgu = '<p class="warn">WARNING: turbo enabled, <span>&nbsp;client may not detect and resume incomplete uploads; see turbo-button tooltip</span></p>',
-            msgs = '<p class="warn">WARNING: turbo enabled, <span>&nbsp;search may give false-positives; see turbo-button tooltip</span></p>',
+            msgs = '<p class="warn">WARNING: turbo enabled, <span>&nbsp;search results can be incorrect; see turbo-button tooltip</span></p>',
             msg = fsearch ? msgs : msgu,
             omsg = fsearch ? msgu : msgs,
             html = ebi('u2foot').innerHTML,
@@ -1687,11 +1828,10 @@ function up2k_init(subtle) {
         catch (ex) { }
 
         try {
-            var fun = fsearch ? 'add' : 'remove',
-                ico = fsearch ? '🔎' : '🚀',
+            var ico = fsearch ? '🔎' : '🚀',
                 desc = fsearch ? 'Search' : 'Upload';
 
-            ebi('op_up2k').classList[fun]('srch');
+            clmod(ebi('op_up2k'), 'srch', fsearch);
             ebi('u2bm').innerHTML = ico + ' <sup>' + desc + '</sup>';
         }
         catch (ex) { }
@@ -1712,11 +1852,14 @@ function up2k_init(subtle) {
                 flag = up2k_flagbus();
             }
             catch (ex) {
-                console.log("flag error: " + ex.toString());
+                toast.err(5, "not supported on your browser:\n" + ex);
                 tgl_flag_en();
             }
         }
         else if (!flag_en && flag) {
+            if (flag.ours)
+                flag.give();
+
             flag.ch.close();
             flag = false;
         }
@@ -1747,9 +1890,9 @@ function up2k_init(subtle) {
     if (o)
         o.addEventListener('click', tgl_fsearch, false);
 
-    var nodes = ebi('u2conf').getElementsByTagName('a');
-    for (var a = nodes.length - 1; a >= 0; a--)
-        nodes[a].addEventListener('touchend', nop, false);
+    ebi('u2etas').onclick = function () {
+        clmod(ebi('u2etas'), 'o', 't');
+    };
 
     set_fsearch();
     bumpthread({ "target": 1 });
